﻿using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Windows.Forms;

using TestCommon;
using TestClient.DataService;

namespace TestClient
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 임시 디렉토리 경로
        /// </summary>
        private const string TEMPORARY_DIRECTORY_PATH = "d:\\temporary_client";

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.getHTTPDataButton.Click += getHTTPDataButton_Click;
            this.getTCPDataButton.Click  += getTCPDataButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region HTTP 데이터 구하기 버튼 클릭시 처리하기 - getHTTPDataButton_Click(sender, e)

        /// <summary>
        /// HTTP 데이터 구하기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void getHTTPDataButton_Click(object sender, EventArgs e)
        {
            WriteMessage("START GET HTTP DATA");

            string dataID        = "0001";
            string conditionData = JSONHelper.Serialize((int)1000000);

            List<Student> list = GetData<List<Student>>("BasicHttpBinding_IDataService", dataID, conditionData);

            string message;

            if(list != null)
            {
                message = list.Count.ToString();
            }
            else
            {
                message = "데이터가 없습니다.";
            }

            WriteMessage("END GET HTTP DATA");

            MessageBox.Show(message);
        }

        #endregion
        #region TCP 데이터 구하기 버튼 클릭시 처리하기 - getTCPDataButton_Click(sender, e)

        /// <summary>
        /// TCP 데이터 구하기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void getTCPDataButton_Click(object sender, EventArgs e)
        {
            WriteMessage("START GET TCP DATA");

            string dataID        = "0001";
            string conditionData = JSONHelper.Serialize((int)1000000);

            List<Student> list = GetData<List<Student>>("NetTcpBinding_IDataService", dataID, conditionData);

            string message;

            if(list != null)
            {
                message = list.Count.ToString();
            }
            else
            {
                message = "데이터가 없습니다.";
            }

            WriteMessage("END GET TCP DATA");

            MessageBox.Show(message);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 데이터 구하기 - GetData<TResult>(endpointConfigurationName, dataID, conditionData)

        /// <summary>
        /// 데이터 구하기
        /// </summary>
        /// <typeparam name="TResult">결과 타입</typeparam>
        /// <param name="endpointConfigurationName">종점 구성명</param>
        /// <param name="dataID">데이터 ID</param>
        /// <param name="conditionData">조건 데이터</param>
        /// <returns>데이터</returns>
        private TResult GetData<TResult>(string endpointConfigurationName, string dataID, string conditionData)
        {
            string transactionID = GUIDHelper.GetGUID();

            if(!Directory.Exists(TEMPORARY_DIRECTORY_PATH))
            {
                Directory.CreateDirectory(TEMPORARY_DIRECTORY_PATH);
            }

            string transactionFilePath = Path.Combine(TEMPORARY_DIRECTORY_PATH, string.Format("{0}.dat", transactionID));

            try
            {
                using(DataServiceClient client = new DataServiceClient(endpointConfigurationName))
                {
                    using(FileStream fileStream = new FileStream(transactionFilePath, FileMode.Create, FileAccess.Write))
                    {
                        using(Stream stream = client.GetData(dataID, conditionData))
                        {
                            stream.CopyTo(fileStream);
                        }
                    }
                }

                TResult result = default(TResult);

                using(FileStream fileStream = new FileStream(transactionFilePath, FileMode.Open, FileAccess.Read, FileShare.Read))
                {
                    using(GZipStream gZipStream = new GZipStream(fileStream, CompressionMode.Decompress))
                    {
                        result = JSONHelper.Deserialize<TResult>(gZipStream);
                    }
                }

                return result;
            }
            finally
            {
                try
                {
                    File.Delete(transactionFilePath);
                }
                catch
                {
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 메시지 쓰기 - WriteMessage(format, parameterArray)

        /// <summary>
        /// 메시지 쓰기
        /// </summary>
        /// <param name="format">포맷 문자열</param>
        /// <param name="parameterArray">매개 변수 배열</param>
        private void WriteMessage(string format, params object[] parameterArray)
        {
            string message;

            if(parameterArray.Length == 0)
            {
                message = format;
            }
            else
            {
                message = string.Format(format, parameterArray);
            }

            this.listBox.Items.Add(string.Format("[{0}] {1}", DateTime.Now.ToString("HH:mm:ss"), message));

            this.listBox.Update();
        }

        #endregion
    }
}