﻿using System.IO;

using Newtonsoft.Json;

namespace TestCommon
{
    /// <summary>
    /// JSON 헬퍼
    /// </summary>
    public static class JSONHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 직렬화 하기 - Serialize(sourceObject)

        /// <summary>
        /// 직렬화 하기
        /// </summary>
        /// <param name="sourceObject">소스 객체</param>
        /// <returns>JSON 문자열</returns>
        public static string Serialize(object sourceObject)
        {
            string targetJSON = JsonConvert.SerializeObject(sourceObject);

            return targetJSON;
        }

        #endregion
        #region 직렬화 하기 - Serialize(sourceObject, targetStream)

        /// <summary>
        /// 직렬화 하기
        /// </summary>
        /// <param name="sourceObject">소스 객체</param>
        /// <param name="targetStream">타겟 스트림</param>
        public static void Serialize(object sourceObject, Stream targetStream)
        {
            using(StreamWriter streamWriter = new StreamWriter(targetStream))
            {
                using(JsonTextWriter jsonTextWriter = new JsonTextWriter(streamWriter))
                {
                    JsonSerializer jsonSerializer = new JsonSerializer();

                    jsonSerializer.Serialize(jsonTextWriter, sourceObject);

                    jsonTextWriter.Flush();
                }
            }
        }

        #endregion

        #region 역직렬화 하기 - Deserialize<T>(json)

        /// <summary>
        /// 역직렬화 하기
        /// </summary>
        /// <typeparam name="T">타입</typeparam>
        /// <param name="json">JSON</param>
        /// <returns>객체</returns>
        public static T Deserialize<T>(string json)
        {
            T targetObject = JsonConvert.DeserializeObject<T>(json);

            return targetObject;
        }

        #endregion
        #region 역직렬화 하기 - Deserialize<T>(sourceStream)

        /// <summary>
        /// 역직렬화 하기
        /// </summary>
        /// <typeparam name="T">타입</typeparam>
        /// <param name="sourceStream">소스 스트림</param>
        /// <returns>역직렬화 하기</returns>
        public static T Deserialize<T>(Stream sourceStream)
        {
            using(StreamReader streamReader = new StreamReader(sourceStream))
            {
                using(JsonTextReader jsonTextReader = new JsonTextReader(streamReader))
                {
                    JsonSerializer jsonSerializer = new JsonSerializer();

                    T targetObject = jsonSerializer.Deserialize<T>(jsonTextReader);

                    return targetObject;
                }
            }
        }

        #endregion
    }
}